// Copyright (C) 2008, 2009 by Florent Lamiraux, Thomas Moulard, CNRS.
//
// This file is part of the hpp-walkfootplanner.
//
// This software is provided "as is" without warranty of any kind,
// either expressed or implied, including but not limited to the
// implied warranties of fitness for a particular purpose.
//
// See the COPYING file for more information.

#ifndef HPPWALKFOOTPLANNER_FOOTSTEP_SEQUENCE_BUILDER_HH
# define HPPWALKFOOTPLANNER_FOOTSTEP_SEQUENCE_BUILDER_HH
# include <utility>

# include <boost/function.hpp>

# include <hpp/walkfootplanner/config.hh>
# include <hpp/walkfootplanner/footstep-sequence.hh>
# include <hpp/walkfootplanner/fwd.hh>

namespace hpp
{
  namespace walkFootPlanner
  {
    /// \brief Default conversion from DOFs to footstep.
    ///
    /// Require a 3 degrees of freedom, respectively (X, Y, theta)
    /// and map it into a footStep_t (i.e. 3d uBLAS vector).
    HPP_WALKFOOTPLANNER_DLLAPI footStep_t
    convertToFootStepXYTheta (CkwsConfigShPtr cfg);

    class HPP_WALKFOOTPLANNER_DLLAPI FootStepSequenceBuilder
    {
    public:
      typedef boost::function<footStep_t (CkwsConfigShPtr)> footStepGetter_t;
      FootStepSequenceBuilder (footStepGetter_t footStepGetter
			       = &convertToFootStepXYTheta);
      ~FootStepSequenceBuilder ();

      void operator() (CkwsPathShPtr path,
		       FootStepSequence& footStepSequence,
		       const Planner& planner);

    private:
      const footStepGetter_t footStepGetter_;
    };
  } // end of namespace walkFootPlanner.
} // end of namespace hpp.

#endif //! HPPWALKFOOTPLANNER_FOOTSTEP_SEQUENCE_BUILDER_HH
