// Copyright (C) 2008, 2009 by Florent Lamiraux, Thomas Moulard, CNRS.
//
// This file is part of the hpp-walkfootplanner.
//
// This software is provided "as is" without warranty of any kind,
// either expressed or implied, including but not limited to the
// implied warranties of fitness for a particular purpose.
//
// See the COPYING file for more information.

#ifndef HPPWALKFOOTPLANNER_MINIMIZETIME_HH
# define HPPWALKFOOTPLANNER_MINIMIZETIME_HH
# include <boost/mpl/vector.hpp>

# include <roboptim/core/derivable-function.hh>
# include <roboptim/core/function.hh>
# include <roboptim/core/linear-function.hh>
# include <roboptim/core/numeric-linear-function.hh>
# include <roboptim/core/solver.hh>
# include <roboptim/core/visualization/fwd.hh>

# include <roboptim/trajectory/free-time-trajectory.hh>
# include <roboptim/trajectory/spline-length.hh>

# include <hpp/model/body.hh>

# include <hpp/walkfootplanner/spline-directpath.hh>
# include <hpp/walkfootplanner/config.hh>

namespace hpp
{
  namespace walkFootPlanner
  {
    class HPP_WALKFOOTPLANNER_DLLAPI MinimizeTime
    {
    public:
      typedef roboptim::LinearFunction linearFunction_t;
      typedef roboptim::DerivableFunction derivableFunction_t;
      typedef roboptim::CubicBSpline spline_t;
      typedef roboptim::FreeTimeTrajectory<spline_t> freeTime_t;
      typedef roboptim::Function::interval_t interval_t;
      typedef roboptim::NumericLinearFunction numericLinearFunction_t;

      typedef boost::mpl::vector<linearFunction_t, derivableFunction_t> clist_t;
      typedef roboptim::Solver<derivableFunction_t, clist_t> solver_t;
      typedef solver_t::problem_t problem_t;

      MinimizeTime (const spline_t& spline,
		    model::BodyShPtr body,
		    SplineDirectPathShPtr directPath,
		    double minDistance,
		    double stepSpace,
		    const double& vMaxX,
		    const double& vMaxY,
		    const double& vMaxTheta) throw ();
      ~MinimizeTime () throw ();

      roboptim::visualization::Gnuplot
      display (SplineDirectPathShPtr path) const throw ();

      const problem_t& getProblem () const throw ();

    private:
      freeTime_t freeTimeTraj_;
      numericLinearFunction_t cost_;
      problem_t problem_;
      double stepSpace_;
    };
  } // end of namespace walkFootPlanner.
} // end of namespace hpp.

#endif //! HPPWALKFOOTPLANNER_MINIMIZETIME_HH
