// Copyright (C) 2008, 2009 by Florent Lamiraux, Thomas Moulard, CNRS.
//
// This file is part of the hpp-walkfootplanner.
//
// This software is provided "as is" without warranty of any kind,
// either expressed or implied, including but not limited to the
// implied warranties of fitness for a particular purpose.
//
// See the COPYING file for more information.

#ifndef HPPWALKFOOTPLANNER_OBSTACLE_CONSTRAINT_HH
# define HPPWALKFOOTPLANNER_OBSTACLE_CONSTRAINT_HH

# include <kcd2/kcdPoint.h>
# include <roboptim/core/derivable-function.hh>
# include <roboptim/trajectory/stable-time-point.hh>

# include <hpp/walkfootplanner/spline-directpath.hh>

namespace hpp
{
  namespace walkFootPlanner
  {
    struct HPP_WALKFOOTPLANNER_DLLAPI DistAndPairsOfPointsResult
    {
      double distance;
      CkcdPoint pointBody;
      CkcdPoint pointEnv;
    };

    struct HPP_WALKFOOTPLANNER_DLLAPI LtCkwsConfig
    {
    public:
      bool operator () (const CkwsConfig& l, const CkwsConfig& r);
    };

    struct HPP_WALKFOOTPLANNER_DLLAPI DistanceProxy
    {
    public:
      typedef std::map<CkwsConfig, DistAndPairsOfPointsResult, LtCkwsConfig> cache_t;
      explicit DistanceProxy (unsigned obstacle,
			      model::DeviceShPtr device,
			      model::BodyShPtr body);
      DistAndPairsOfPointsResult operator () (CkwsConfig&);
    private:
      unsigned obstacle_;
      model::DeviceShPtr device_;
      model::BodyShPtr body_;
      cache_t cache_;
    };



    /// Implement robot-obstacle interactions nonholonomic trajectory deformation.
    class HPP_WALKFOOTPLANNER_DLLAPI RobotObstacleDistance : public roboptim::DerivableFunction
    {
    public:
      RobotObstacleDistance (SplineDirectPathShPtr dp,
			     roboptim::StableTimePoint tpt,
			     size_type obstacleId) throw ();

      ~RobotObstacleDistance () throw ();

    protected:
      void impl_compute (result_t& res, const argument_t& p) const throw ();
      void impl_gradient (gradient_t& grad, const argument_t& p, size_type)
	const throw ();

    private:
      SplineDirectPathShPtr dp_;
      roboptim::StableTimePoint tpt_;
      size_type obstacleId_;

      /// \brief Direct path's device.
      model::DeviceShPtr hppDevice_;

      /// \brief Body attached to the root joint.
      model::BodyShPtr hppBody_;

      /// \brief Interval variable used to store a temporary configuration.
      mutable CkwsConfig cfg_;

      /// \brief Cache distance computations.
      mutable DistanceProxy distanceProxy_;
    };
  } // end of namespace walkFootPlanner.
} // end of namespace hpp.

#endif //! HPPWALKFOOTPLANNER_OBSTACLE_CONSTRAINT_HH
